searchData={"items":[{"type":"module","title":"Finch","doc":"An HTTP client with a focus on performance, built on top of\n[Mint](https://github.com/elixir-mint/mint) and [NimblePool](https://github.com/dashbitco/nimble_pool).\n\nWe attempt to achieve this goal by providing efficient connection pooling strategies and avoiding copying of memory wherever possible.\n\nMost developers will most likely prefer to use the fabulous HTTP client [Req](https://github.com/wojtekmach/req) which takes advantage of Finch's pooling and provides an extremely friendly and pleasant to use API.","ref":"Finch.html"},{"type":"module","title":"Usage - Finch","doc":"In order to use Finch, you must start it and provide a `:name`. Often in your\nsupervision tree:\n\n```elixir\nchildren = [\n  {Finch, name: MyFinch}\n]\n```\n\nOr, in rare cases, dynamically:\n\n```elixir\nFinch.start_link(name: MyFinch)\n```\n\nOnce you have started your instance of Finch, you are ready to start making requests:\n\n```elixir\nFinch.build(:get, \"https://hex.pm\") |> Finch.request(MyFinch)\n```\n\nWhen using HTTP/1, Finch will parse the passed in URL into a `{scheme, host, port}`\ntuple, and maintain one or more connection pools for each `{scheme, host, port}` you\ninteract with.\n\nYou can also configure a pool size and count to be used for specific URLs that are\nknown before starting Finch. The passed URLs will be parsed into `{scheme, host, port}`,\nand the corresponding pools will be started. See `Finch.start_link/1` for configuration\noptions.\n\n```elixir\nchildren = [\n  {Finch,\n   name: MyConfiguredFinch,\n   pools: %{\n     :default => [size: 10],\n     \"https://hex.pm\" => [size: 32, count: 8]\n   }}\n]\n```\n\nPools will be started for each configured `{scheme, host, port}` when Finch is started.\nFor any unconfigured `{scheme, host, port}`, the pool will be started the first time\nit is requested. Note pools are not automatically terminated by default, if you need to\nterminate them after some idle time, use the `pool_max_idle_time` option (available only for HTTP1 pools).","ref":"Finch.html#module-usage"},{"type":"module","title":"Telemetry - Finch","doc":"Finch uses Telemetry to provide instrumentation. See the `Finch.Telemetry`\nmodule for details on specific events.","ref":"Finch.html#module-telemetry"},{"type":"module","title":"Logging TLS Secrets - Finch","doc":"Finch supports logging TLS secrets to a file. These can be later used in a tool such as\nWireshark to decrypt HTTPS sessions. To use this feature you must specify the file to\nwhich the secrets should be written. If you are using TLSv1.3 you must also add\n`keep_secrets: true` to your pool `:transport_opts`. For example:\n\n```elixir\n{Finch,\n name: MyFinch,\n pools: %{\n   default: [conn_opts: [transport_opts: [keep_secrets: true]]]\n }}\n```\n\nThere are two different ways to specify this file:\n\n1. The `:ssl_key_log_file` connection option in your pool configuration. For example:\n\n```elixir\n{Finch,\n name: MyFinch,\n pools: %{\n   default: [\n     conn_opts: [\n       ssl_key_log_file: \"/writable/path/to/the/sslkey.log\"\n     ]\n   ]\n }}\n```\n\n2. Alternatively, you could also set the `SSLKEYLOGFILE` environment variable.","ref":"Finch.html#module-logging-tls-secrets"},{"type":"function","title":"Finch.async_request/3","doc":"Sends an HTTP request asynchronously, returning a request reference.\n\nIf the request is sent using HTTP1, an extra process is spawned to\nconsume messages from the underlying socket. The messages are sent\nto the current process as soon as they arrive, as a firehose.  If\nyou wish to maximize request rate or have more control over how\nmessages are streamed, a strategy using `request/3` or `stream/5`\nshould be used instead.","ref":"Finch.html#async_request/3"},{"type":"function","title":"Receiving the response - Finch.async_request/3","doc":"Response information is sent to the calling process as it is received\nin `{ref, response}` tuples.\n\nIf the calling process exits before the request has completed, the\nrequest will be canceled.\n\nResponses include:\n\n  * `{:status, status}` - HTTP response status\n  * `{:headers, headers}` - HTTP response headers\n  * `{:data, data}` - section of the HTTP response body\n  * `{:error, exception}` - an error occurred during the request\n  * `:done` - request has completed successfully\n\nOn a successful request, a single `:status` message will be followed\nby a single `:headers` message, after which more than one `:data`\nmessages may be sent. If trailing headers are present, a final\n`:headers` message may be sent. Any `:done` or `:error` message\nindicates that the request has succeeded or failed and no further\nmessages are expected.","ref":"Finch.html#async_request/3-receiving-the-response"},{"type":"function","title":"Example - Finch.async_request/3","doc":"iex> req = Finch.build(:get, \"https://httpbin.org/stream/5\")\n    iex> ref = Finch.async_request(req, MyFinch)\n    iex> flush()\n    {ref, {:status, 200}}\n    {ref, {:headers, [...]}}\n    {ref, {:data, \"...\"}}\n    {ref, :done}","ref":"Finch.html#async_request/3-example"},{"type":"function","title":"Options - Finch.async_request/3","doc":"Shares options with `request/3`.","ref":"Finch.html#async_request/3-options"},{"type":"function","title":"Finch.build/5","doc":"Builds an HTTP request to be sent with `request/3` or `stream/4`.\n\nIt is possible to send the request body in a streaming fashion. In order to do so, the\n`body` parameter needs to take form of a tuple `{:stream, body_stream}`, where `body_stream`\nis a `Stream`.","ref":"Finch.html#build/5"},{"type":"function","title":"Finch.cancel_async_request/1","doc":"Cancels a request sent with `async_request/3`.","ref":"Finch.html#cancel_async_request/1"},{"type":"function","title":"Finch.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Finch.html#child_spec/1"},{"type":"function","title":"Finch.get_pool_status/2","doc":"Get pool metrics list.\n\nThe number of items present on the metrics list depends on the `:count` option\neach metric will have a `pool_index` going from 1 to `:count`.\n\nThe metrics struct depends on the pool scheme defined on the `:protocols` option\n`Finch.HTTP1.PoolMetrics` for `:http1` and `Finch.HTTP2.PoolMetrics` for `:http2`.\n\nSee the `Finch.HTTP1.PoolMetrics` and `Finch.HTTP2.PoolMetrics` for more details.\n\n`{:error, :not_found}` may return on 2 scenarios:\n  - There is no pool registered for the given pair finch instance and url\n  - The pool is configured with `start_pool_metrics?` option false (default)","ref":"Finch.html#get_pool_status/2"},{"type":"function","title":"Example - Finch.get_pool_status/2","doc":"iex> Finch.get_pool_status(MyFinch, \"https://httpbin.org\")\n    {:ok, [\n      %Finch.HTTP1.PoolMetrics{\n        pool_index: 1,\n        pool_size: 50,\n        available_connections: 43,\n        in_use_connections: 7\n      },\n      %Finch.HTTP1.PoolMetrics{\n        pool_index: 2,\n        pool_size: 50,\n        available_connections: 37,\n        in_use_connections: 13\n      }]\n    }","ref":"Finch.html#get_pool_status/2-example"},{"type":"function","title":"Finch.request/3","doc":"Sends an HTTP request and returns a `Finch.Response` struct.","ref":"Finch.html#request/3"},{"type":"function","title":"Options - Finch.request/3","doc":"* `:pool_timeout` - This timeout is applied when we check out a connection from the pool.\n    Default value is `5_000`.\n\n  * `:receive_timeout` - The maximum time to wait for each chunk to be received before returning an error.\n    Default value is `15_000`.\n\n  * `:request_timeout` - The amount of time to wait for a complete response before returning an error.\n    This timeout only applies to HTTP/1, and its current implementation is a best effort timeout,\n    it does not guarantee the call will return precisely when the time has elapsed.\n    Default value is `:infinity`.","ref":"Finch.html#request/3-options"},{"type":"function","title":"Finch.request!/3","doc":"Sends an HTTP request and returns a `Finch.Response` struct\nor raises an exception in case of failure.\n\nSee `request/3` for more detailed information.","ref":"Finch.html#request!/3"},{"type":"function","title":"Finch.start_link/1","doc":"Start an instance of Finch.","ref":"Finch.html#start_link/1"},{"type":"function","title":"Options - Finch.start_link/1","doc":"* `:name` - The name of your Finch instance. This field is required.\n\n  * `:pools` - A map specifying the configuration for your pools. The keys should be URLs\n  provided as binaries, a tuple `{scheme, {:local, unix_socket}}` where `unix_socket` is the path for\n  the socket, or the atom `:default` to provide a catch-all configuration to be used for any\n  unspecified URLs. See \"Pool Configuration Options\" below for details on the possible map\n  values. Default value is `%{default: [size: 50, count: 1]}`.\n\n#","ref":"Finch.html#start_link/1-options"},{"type":"function","title":"Pool Configuration Options - Finch.start_link/1","doc":"* `:protocol`\n\n* `:protocols` - The type of connections to support.\n\n  If using `:http1` only, an HTTP1 pool without multiplexing is used. If using `:http2` only, an HTTP2 pool with multiplexing is used. If both are listed, then both HTTP1/HTTP2 connections are supported (via ALPN), but there is no multiplexing.\n\n  The default value is `[:http1]`.\n\n* `:size` (`t:pos_integer/0`) - Number of connections to maintain in each pool. Used only by HTTP1 pools since HTTP2 is able to multiplex requests through a single connection. In other words, for HTTP2, the size is always 1 and the `:count` should be configured in order to increase capacity. The default value is `50`.\n\n* `:count` (`t:pos_integer/0`) - Number of pools to start. HTTP1 pools are able to re-use connections in the same pool and establish new ones only when necessary. However, if there is a high pool count and few requests are made, these requests will be scattered across pools, reducing connection reuse. It is recommended to increase the pool count for HTTP1 only if you are experiencing high checkout times. The default value is `1`.\n\n* `:max_idle_time` (`t:timeout/0`) - The maximum number of milliseconds an HTTP1 connection is allowed to be idle before being closed during a checkout attempt.\n\n* `:conn_opts` (`t:keyword/0`) - These options are passed to `Mint.HTTP.connect/4` whenever a new connection is established. `:mode` is not configurable as Finch must control this setting. Typically these options are used to configure proxying, https settings, or connect timeouts. The default value is `[]`.\n\n* `:pool_max_idle_time` (`t:timeout/0`) - The maximum number of milliseconds that a pool can be idle before being terminated, used only by HTTP1 pools. This options is forwarded to NimblePool and it starts and idle verification cycle that may impact performance if misused. For instance setting a very low timeout may lead to pool restarts. For more information see NimblePool's `handle_ping/2` documentation. The default value is `:infinity`.\n\n* `:conn_max_idle_time` (`t:timeout/0`) - The maximum number of milliseconds an HTTP1 connection is allowed to be idle before being closed during a checkout attempt. The default value is `:infinity`.\n\n* `:start_pool_metrics?` (`t:boolean/0`) - When true, pool metrics will be collected and available through Finch.pool_status/2 The default value is `false`.","ref":"Finch.html#start_link/1-pool-configuration-options"},{"type":"function","title":"Finch.stream/5","doc":"Streams an HTTP request and returns the accumulator.\n\nA function of arity 2 is expected as argument. The first argument\nis a tuple, as listed below, and the second argument is the\naccumulator. The function must return a potentially updated\naccumulator.\n\nSee also `stream_while/5`.\n\n> #","ref":"Finch.html#stream/5"},{"type":"function","title":"HTTP2 streaming and back-pressure {: .warning} - Finch.stream/5","doc":">\n> At the moment, streaming over HTTP2 connections do not provide\n> any back-pressure mechanism: this means the response will be\n> sent to the client as quickly as possible. Therefore, you must\n> not use streaming over HTTP2 for non-terminating responses or\n> when streaming large responses which you do not intend to keep\n> in memory.","ref":"Finch.html#stream/5-http2-streaming-and-back-pressure-warning"},{"type":"function","title":"Stream commands - Finch.stream/5","doc":"* `{:status, status}` - the http response status\n  * `{:headers, headers}` - the http response headers\n  * `{:data, data}` - a streaming section of the http response body\n  * `{:trailers, trailers}` - the http response trailers","ref":"Finch.html#stream/5-stream-commands"},{"type":"function","title":"Options - Finch.stream/5","doc":"Shares options with `request/3`.","ref":"Finch.html#stream/5-options"},{"type":"function","title":"Examples - Finch.stream/5","doc":"path = \"/tmp/archive.zip\"\n    file = File.open!(path, [:write, :exclusive])\n    url = \"https://example.com/archive.zip\"\n    request = Finch.build(:get, url)\n\n    Finch.stream(request, MyFinch, nil, fn\n      {:status, status}, _acc ->\n        IO.inspect(status)\n\n      {:headers, headers}, _acc ->\n        IO.inspect(headers)\n\n      {:data, data}, _acc ->\n        IO.binwrite(file, data)\n    end)\n\n    File.close(file)","ref":"Finch.html#stream/5-examples"},{"type":"function","title":"Finch.stream_while/5","doc":"Streams an HTTP request until it finishes or `fun` returns `{:halt, acc}`.\n\nA function of arity 2 is expected as argument. The first argument\nis a tuple, as listed below, and the second argument is the\naccumulator.\n\nThe function must return:\n\n  * `{:cont, acc}` to continue streaming\n  * `{:halt, acc}` to halt streaming\n\nSee also `stream/5`.\n\n> #","ref":"Finch.html#stream_while/5"},{"type":"function","title":"HTTP2 streaming and back-pressure {: .warning} - Finch.stream_while/5","doc":">\n> At the moment, streaming over HTTP2 connections do not provide\n> any back-pressure mechanism: this means the response will be\n> sent to the client as quickly as possible. Therefore, you must\n> not use streaming over HTTP2 for non-terminating responses or\n> when streaming large responses which you do not intend to keep\n> in memory.","ref":"Finch.html#stream_while/5-http2-streaming-and-back-pressure-warning"},{"type":"function","title":"Stream commands - Finch.stream_while/5","doc":"* `{:status, status}` - the http response status\n  * `{:headers, headers}` - the http response headers\n  * `{:data, data}` - a streaming section of the http response body\n  * `{:trailers, trailers}` - the http response trailers","ref":"Finch.html#stream_while/5-stream-commands"},{"type":"function","title":"Options - Finch.stream_while/5","doc":"Shares options with `request/3`.","ref":"Finch.html#stream_while/5-options"},{"type":"function","title":"Examples - Finch.stream_while/5","doc":"path = \"/tmp/archive.zip\"\n    file = File.open!(path, [:write, :exclusive])\n    url = \"https://example.com/archive.zip\"\n    request = Finch.build(:get, url)\n\n    Finch.stream_while(request, MyFinch, nil, fn\n      {:status, status}, acc ->\n        IO.inspect(status)\n        {:cont, acc}\n\n      {:headers, headers}, acc ->\n        IO.inspect(headers)\n        {:cont, acc}\n\n      {:data, data}, acc ->\n        IO.binwrite(file, data)\n        {:cont, acc}\n    end)\n\n    File.close(file)","ref":"Finch.html#stream_while/5-examples"},{"type":"type","title":"Finch.name/0","doc":"The `:name` provided to Finch in `start_link/1`.","ref":"Finch.html#t:name/0"},{"type":"type","title":"Finch.request_opt/0","doc":"","ref":"Finch.html#t:request_opt/0"},{"type":"type","title":"Finch.request_opts/0","doc":"Options used by request functions.","ref":"Finch.html#t:request_opts/0"},{"type":"opaque","title":"Finch.request_ref/0","doc":"The reference used to identify a request sent using `async_request/3`.","ref":"Finch.html#t:request_ref/0"},{"type":"type","title":"Finch.scheme/0","doc":"","ref":"Finch.html#t:scheme/0"},{"type":"type","title":"Finch.scheme_host_port/0","doc":"","ref":"Finch.html#t:scheme_host_port/0"},{"type":"type","title":"Finch.stream/1","doc":"The stream function given to `stream/5`.","ref":"Finch.html#t:stream/1"},{"type":"type","title":"Finch.stream_while/1","doc":"The stream function given to `stream_while/5`.","ref":"Finch.html#t:stream_while/1"},{"type":"module","title":"Finch.HTTP1.PoolMetrics","doc":"HTTP1 Pool metrics.\n\nAvailable metrics:\n\n  * `:pool_index` - Index of the pool\n  * `:pool_size` - Total number of connections of the pool\n  * `:available_connections` - Number of available connections\n  * `:in_use_connections` - Number of connections currently in use\n\nCaveats:\n\n  * A given number X of `available_connections` does not mean that currently\n  exists X connections to the server sitting on the pool. Because Finch uses \n  a lazy strategy for workers initialization, every pool starts with it's \n  size as available connections even if they are not started yet. In practice\n  this means that `available_connections` may be connections sitting on the pool\n  or available space on the pool for a new one if required.","ref":"Finch.HTTP1.PoolMetrics.html"},{"type":"function","title":"Finch.HTTP1.PoolMetrics.get_pool_status/1","doc":"","ref":"Finch.HTTP1.PoolMetrics.html#get_pool_status/1"},{"type":"function","title":"Finch.HTTP1.PoolMetrics.get_pool_status/3","doc":"","ref":"Finch.HTTP1.PoolMetrics.html#get_pool_status/3"},{"type":"function","title":"Finch.HTTP1.PoolMetrics.init/4","doc":"","ref":"Finch.HTTP1.PoolMetrics.html#init/4"},{"type":"function","title":"Finch.HTTP1.PoolMetrics.maybe_add/2","doc":"","ref":"Finch.HTTP1.PoolMetrics.html#maybe_add/2"},{"type":"type","title":"Finch.HTTP1.PoolMetrics.t/0","doc":"","ref":"Finch.HTTP1.PoolMetrics.html#t:t/0"},{"type":"module","title":"Finch.HTTP2.PoolMetrics","doc":"HTTP2 Pool metrics.\n\nAvailable metrics:\n\n  * `:pool_index` - Index of the pool\n  * `:in_flight_requests` - Number of requests currently on the connection\n\nCaveats:\n\n  * HTTP2 pools have only one connection and leverage the multiplex nature\n  of the protocol. That's why we only keep the in flight requests, representing\n  the number of streams currently running on the connection.","ref":"Finch.HTTP2.PoolMetrics.html"},{"type":"function","title":"Finch.HTTP2.PoolMetrics.get_pool_status/1","doc":"","ref":"Finch.HTTP2.PoolMetrics.html#get_pool_status/1"},{"type":"function","title":"Finch.HTTP2.PoolMetrics.get_pool_status/3","doc":"","ref":"Finch.HTTP2.PoolMetrics.html#get_pool_status/3"},{"type":"function","title":"Finch.HTTP2.PoolMetrics.init/3","doc":"","ref":"Finch.HTTP2.PoolMetrics.html#init/3"},{"type":"function","title":"Finch.HTTP2.PoolMetrics.maybe_add/2","doc":"","ref":"Finch.HTTP2.PoolMetrics.html#maybe_add/2"},{"type":"type","title":"Finch.HTTP2.PoolMetrics.t/0","doc":"","ref":"Finch.HTTP2.PoolMetrics.html#t:t/0"},{"type":"module","title":"Finch.Request","doc":"A request struct.","ref":"Finch.Request.html"},{"type":"function","title":"Finch.Request.put_private/3","doc":"Sets a new **private** key and value in the request metadata. This storage is meant to be used by libraries\nand frameworks to inject information about the request that needs to be retrieved later on, for example,\nfrom handlers that consume `Finch.Telemetry` events.","ref":"Finch.Request.html#put_private/3"},{"type":"type","title":"Finch.Request.body/0","doc":"Optional request body.","ref":"Finch.Request.html#t:body/0"},{"type":"type","title":"Finch.Request.headers/0","doc":"Request headers.","ref":"Finch.Request.html#t:headers/0"},{"type":"type","title":"Finch.Request.method/0","doc":"An HTTP request method represented as an `atom()` or a `String.t()`.\n\nThe following atom methods are supported: `:get`, `:post`, `:put`, `:patch`, `:delete`, `:head`, `:options`.\nYou can use any arbitrary method by providing it as a `String.t()`.","ref":"Finch.Request.html#t:method/0"},{"type":"type","title":"Finch.Request.private_metadata/0","doc":"","ref":"Finch.Request.html#t:private_metadata/0"},{"type":"type","title":"Finch.Request.t/0","doc":"","ref":"Finch.Request.html#t:t/0"},{"type":"type","title":"Finch.Request.url/0","doc":"A Uniform Resource Locator, the address of a resource on the Web.","ref":"Finch.Request.html#t:url/0"},{"type":"module","title":"Finch.Response","doc":"A response to a request.","ref":"Finch.Response.html"},{"type":"type","title":"Finch.Response.t/0","doc":"","ref":"Finch.Response.html#t:t/0"},{"type":"module","title":"Finch.Telemetry","doc":"Telemetry integration.\n\nUnless specified, all times are in `:native` units.\n\nFinch executes the following events:\n\n#","ref":"Finch.Telemetry.html"},{"type":"module","title":"Request Start - Finch.Telemetry","doc":"`[:finch, :request, :start]` - Executed when `Finch.request/3` or `Finch.stream/5` is called.\n\n##","ref":"Finch.Telemetry.html#module-request-start"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:system_time` - The system time.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:request` - The request (`Finch.Request`).\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Request Stop - Finch.Telemetry","doc":"`[:finch, :request, :stop]` - Executed after `Finch.request/3` or `Finch.stream/5` ended.\n\n##","ref":"Finch.Telemetry.html#module-request-stop"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - Time taken from the request start event.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n* `:request` - The request (`Finch.Request`).\n* `:result` - The result of the operation. In case of `Finch.stream/5` this is\n  `{:ok, acc} | {:error, Exception.t()}`, where `acc` is the accumulator result of the\n  reducer passed in `Finch.stream/5`. In case of `Finch.request/3` this is\n  `{:ok, Finch.Response.t()} | {:error, Exception.t()}`.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Request Exception - Finch.Telemetry","doc":"`[:finch, :request, :exception]` - Executed when an exception occurs while executing\n  `Finch.request/3` or `Finch.stream/5`.\n\n##","ref":"Finch.Telemetry.html#module-request-exception"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - The time it took since the start before raising the exception.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:request` - The request (`Finch.Request`).\n  * `:kind` - The type of exception.\n  * `:reason` - Error description or error data.\n  * `:stacktrace` - The stacktrace.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Queue Start - Finch.Telemetry","doc":"`[:finch, :queue, :start]` - Executed before checking out an HTTP1 connection from the pool.\n\n##","ref":"Finch.Telemetry.html#module-queue-start"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:system_time` - The system time.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:pool` - The pool's PID.\n  * `:request` - The request (`Finch.Request`).\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Queue Stop - Finch.Telemetry","doc":"`[:finch, :queue, :stop]` - Executed after an HTTP1 connection is retrieved from the pool.\n\n##","ref":"Finch.Telemetry.html#module-queue-stop"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - Time taken to check out a pool connection.\n  * `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:pool` - The pool's PID.\n  * `:request` - The request (`Finch.Request`).\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Queue Exception - Finch.Telemetry","doc":"`[:finch, :queue, :exception]` - Executed if checking out an HTTP1 connection throws an exception.\n\n##","ref":"Finch.Telemetry.html#module-queue-exception"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - The time it took since queue start event before raising an exception.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:request` - The request (`Finch.Request`).\n  * `:kind` - The type of exception.\n  * `:reason` - Error description or error data.\n  * `:stacktrace` - The stacktrace.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Connect Start - Finch.Telemetry","doc":"`[:finch, :connect, :start]` - Executed before opening a new connection.\n  If a connection is being re-used this event will *not* be executed.\n\n##","ref":"Finch.Telemetry.html#module-connect-start"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:system_time` - The system time.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:scheme` - The scheme used in the connection. either `http` or `https`.\n  * `:host` - The host address.\n  * `:port` - The port to connect on.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Connect Stop - Finch.Telemetry","doc":"`[:finch, :connect, :stop]` - Executed after a connection is opened.\n\n##","ref":"Finch.Telemetry.html#module-connect-stop"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - Time taken to connect to the host.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:scheme` - The scheme used in the connection. either `http` or `https`.\n  * `:host` - The host address.\n  * `:port` - The port to connect on.\n  * `:error` - This value is optional. It includes any errors that occurred while opening the connection.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Send Start - Finch.Telemetry","doc":"`[:finch, :send, :start]` - Executed before sending a request.\n\n##","ref":"Finch.Telemetry.html#module-send-start"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:system_time` - The system time.\n  * `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:request` - The request (`Finch.Request`).\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Send Stop - Finch.Telemetry","doc":"`[:finch, :send, :stop]` - Executed after a request is finished.\n\n##","ref":"Finch.Telemetry.html#module-send-stop"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:duration` - Time taken to make the request.\n  * `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:request` - The request (`Finch.Request`).\n  * `:error` - This value is optional. It includes any errors that occurred while making the request.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Receive Start - Finch.Telemetry","doc":"`[:finch, :recv, :start]` - Executed before receiving the response.\n\n##","ref":"Finch.Telemetry.html#module-receive-start"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:system_time` - The system time.\n  * `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:request` - The request (`Finch.Request`).\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Receive Stop - Finch.Telemetry","doc":"`[:finch, :recv, :stop]` - Executed after a response has been fully received.\n\n##","ref":"Finch.Telemetry.html#module-receive-stop"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - Duration to receive the response.\n  * `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:request` - The request (`Finch.Request`).\n  * `:status` - The response status (`Mint.Types.status()`).\n  * `:headers` - The response headers (`Mint.Types.headers()`).\n  * `:error` - This value is optional. It includes any errors that occurred while receiving the response.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Receive Exception - Finch.Telemetry","doc":"`[:finch, :recv, :exception]` - Executed if an exception is thrown before the response has\n  been fully received.\n\n##","ref":"Finch.Telemetry.html#module-receive-exception"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:duration` - The time it took before raising an exception\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:request` - The request (`Finch.Request`).\n  * `:kind` - The type of exception.\n  * `:reason` - Error description or error data.\n  * `:stacktrace` - The stacktrace.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Reused Connection - Finch.Telemetry","doc":"`[:finch, :reused_connection]` - Executed if an existing HTTP1 connection is reused. There are no measurements provided with this event.\n\n##","ref":"Finch.Telemetry.html#module-reused-connection"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:name` - The name of the Finch instance.\n  * `:scheme` - The scheme used in the connection. either `http` or `https`.\n  * `:host` - The host address.\n  * `:port` - The port to connect on.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Conn Max Idle Time Exceeded - Finch.Telemetry","doc":"`[:finch, :conn_max_idle_time_exceeded]` - Executed if an HTTP1 connection was discarded because the `conn_max_idle_time` had been reached.\n\n##","ref":"Finch.Telemetry.html#module-conn-max-idle-time-exceeded"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:scheme` - The scheme used in the connection. either `http` or `https`.\n  * `:host` - The host address.\n  * `:port` - The port to connect on.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Pool Max Idle Time Exceeded - Finch.Telemetry","doc":"`[:finch, :pool_max_idle_time_exceeded]` - Executed if an HTTP1 pool was terminated because the `pool_max_idle_time` has been reached. There are no measurements provided with this event.\n\n##","ref":"Finch.Telemetry.html#module-pool-max-idle-time-exceeded"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:scheme` - The scheme used in the connection. either `http` or `https`.\n  * `:host` - The host address.\n  * `:port` - The port to connect on.\n\n#","ref":"Finch.Telemetry.html#module-metadata"},{"type":"module","title":"Max Idle Time Exceeded (Deprecated) - Finch.Telemetry","doc":"`[:finch, :max_idle_time_exceeded]` - Executed if an HTTP1 connection was discarded because the `max_idle_time` had been reached.\n\n*Deprecated:* use `:conn_max_idle_time_exceeded` event instead.\n\n##","ref":"Finch.Telemetry.html#module-max-idle-time-exceeded-deprecated"},{"type":"module","title":"Measurements - Finch.Telemetry","doc":"* `:idle_time` - Elapsed time since the connection was last checked in or initialized.\n\n##","ref":"Finch.Telemetry.html#module-measurements"},{"type":"module","title":"Metadata - Finch.Telemetry","doc":"* `:scheme` - The scheme used in the connection. either `http` or `https`.\n  * `:host` - The host address.\n  * `:port` - The port to connect on.","ref":"Finch.Telemetry.html#module-metadata"},{"type":"exception","title":"Finch.Error","doc":"An HTTP error.\n\nThis exception struct is used to represent errors of all sorts for the HTTP/2 protocol.","ref":"Finch.Error.html"},{"type":"type","title":"Finch.Error.t/0","doc":"","ref":"Finch.Error.html#t:t/0"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"v0.19.0 (2024-09-04) - Changelog","doc":"#","ref":"changelog.html#v0-19-0-2024-09-04"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Update @mint_tls_opts in pool_manager.ex #266\n- Document there is no backpressure on HTTP2 #283\n- Fix test: compare file size instead of map #284\n- Finch.request/3: Use improper list and avoid Enum.reverse #286\n- Require Mint 1.6 #287\n- Remove castore dependency #274\n- Fix typos and improve language in docs and comments #285\n- fix logo size in README #275\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Tweak Finch supervisor children startup order #289, fixes #277\n- implement handle_cancelled/2 pool callback #268, fixes #257\n- type Finch.request_opt() was missing the :request_timeout option #278","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.18.0 (2024-02-09) - Changelog","doc":"#","ref":"changelog.html#v0-18-0-2024-02-09"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add Finch name to telemetry events #252\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- Fix several minor dialyzer errors and run dialyzer in CI #259, #261","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"v0.17.0 (2024-01-07) - Changelog","doc":"#","ref":"changelog.html#v0-17-0-2024-01-07"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add support for async requests #228, #231\n- Add stream example to docs #230\n- Fix calls to deprecated Logger.warn/2 #232\n- Fix typos #233\n- Docs: do not use streams with async_request #238\n- Add Finch.stream_while/5 #239\n- Set MIX_ENV=test on CI #241\n- Update HTTP/2 pool log level to warning for retried action #240\n- Split trailers from headers #242\n- Introduce :request_timeout option #244\n- Support ALPN over HTTP1 pools #250\n- Deprecate :protocol in favour of :protocols #251\n- Implement pool telemetry #248","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.16.0 (2023-04-13) - Changelog","doc":"#","ref":"changelog.html#v0-16-0-2023-04-13"},{"type":"extras","title":"Enhancements - Changelog","doc":"- add `Finch.request!/3` #219\n- allow usage with nimble_pool 1.0 #220","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.15.0 (2023-03-16) - Changelog","doc":"#","ref":"changelog.html#v0-15-0-2023-03-16"},{"type":"extras","title":"Enhancements - Changelog","doc":"- allow usage with nimble_options 1.0 #218\n- allow usage with castore 1.0 #210","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.14.0 (2022-11-30) - Changelog","doc":"#","ref":"changelog.html#v0-14-0-2022-11-30"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Improve error message for pool timeouts #126\n- Relax nimble_options version to allow usage with 0.5.0 #204","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.13.0 (2022-07-26) - Changelog","doc":"#","ref":"changelog.html#v0-13-0-2022-07-26"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Define `Finch.child_spec/1` which will automatically use the `Finch` `:name` as the `:id`, allowing users to start multiple instances under the same Supervisor without any additional configuration #202\n- Include the changelog in the generated HexDocs #201\n- Fix typo in `Finch.Telemetry` docs #198","ref":"changelog.html#enhancements"},{"type":"extras","title":"v0.12.0 (2022-05-03) - Changelog","doc":"#","ref":"changelog.html#v0-12-0-2022-05-03"},{"type":"extras","title":"Enhancements - Changelog","doc":"- Add support for private request metadata #180\n- Hide docs for deprecated `Finch.request/6` #195\n- Add support for Mint.UnsafeProxy connections #184\n\n#","ref":"changelog.html#enhancements"},{"type":"extras","title":"Bug Fixes - Changelog","doc":"- In v0.11.0 headers and status codes were added to Telemetry events in a way that made invalid assumptions\n  regarding the shape of the response accumulator, this has been resolved in #196\n\n#","ref":"changelog.html#bug-fixes"},{"type":"extras","title":"Breaking Changes - Changelog","doc":"- Telemetry updates #176\n  - Rename the telemetry event `:request` to `:send` and `:response` to `:recv`.\n  - Introduce a new `:request` field which contains the full `Finch.Request.t()` in place of the `:scheme`, `:host`, `:port`, `:path`, `:method` fields wherever possible. The new `:request` field can be found on the `:request`, `:queue`, `:send`, and `:recv` events.\n  - Rename the meta data field `:error` to `:reason` for all `:exception` events to follow the standard introduced in [telemetry](https://github.com/beam-telemetry/telemetry/blob/3f069cfd2193396bee221d0709287c1bdaa4fabf/src/telemetry.erl#L335)\n  - Introduce a new `[:finch, :request, :start | :stop | :exception]` telemetry event that emits\n    whenever `Finch.request/3` or `Finch.stream/5` are called.","ref":"changelog.html#breaking-changes"},{"type":"extras","title":"v0.11.0 (2022-03-28) - Changelog","doc":"- Add `:pool_max_idle_time` option to enable termination of idle HTTP/1 pools.\n- Add `:conn_max_idle_time` and deprecate `:max_idle_time` to make the distinction from\n  `:pool_max_idle_time` more obvious.\n- Add headers and status code to Telemetry events.","ref":"changelog.html#v0-11-0-2022-03-28"},{"type":"extras","title":"v0.10.2 (2022-01-12) - Changelog","doc":"- Complete the typespec for Finch.Request.t()\n- Fix the typespec for Finch.build/5\n- Update deps","ref":"changelog.html#v0-10-2-2022-01-12"},{"type":"extras","title":"v0.10.1 (2021-12-27) - Changelog","doc":"- Fix handling of iodata in HTTP/2 request streams.","ref":"changelog.html#v0-10-1-2021-12-27"},{"type":"extras","title":"v0.10.0 (2021-12-12) - Changelog","doc":"- Add ability to stream the request body for HTTP/2 requests.\n- Check and respect window sizes during HTTP/2 requests.","ref":"changelog.html#v0-10-0-2021-12-12"},{"type":"extras","title":"v0.9.1 (2021-10-17) - Changelog","doc":"- Upgrade NimbleOptions dep to 0.4.0.","ref":"changelog.html#v0-9-1-2021-10-17"},{"type":"extras","title":"v0.9.0 (2021-10-17) - Changelog","doc":"- Add support for unix sockets.","ref":"changelog.html#v0-9-0-2021-10-17"},{"type":"extras","title":"v0.8.3 (2021-10-15) - Changelog","doc":"- Return Error struct when HTTP2 connection is closed and a timeout occurs.\n- Do not leak messages/connections when cancelling streaming requests.","ref":"changelog.html#v0-8-3-2021-10-15"},{"type":"extras","title":"v0.8.2 (2021-09-09) - Changelog","doc":"- Demonitor http/2 connections when the request is done.","ref":"changelog.html#v0-8-2-2021-09-09"},{"type":"extras","title":"v0.8.1 (2021-07-27) - Changelog","doc":"- Update mix.exs to allow compatibility with Telemetry v1.0\n- Avoid appending \"?\" to request_path when query string is an empty string","ref":"changelog.html#v0-8-1-2021-07-27"},{"type":"extras","title":"v0.8.0 (2021-06-23) - Changelog","doc":"- HTTP2 connections will now always return Exceptions.","ref":"changelog.html#v0-8-0-2021-06-23"},{"type":"extras","title":"v0.7.0 (2021-05-10) - Changelog","doc":"- Add support for SSLKEYLOGFILE.\n- Drop HTTPS options for default HTTP pools to avoid `:badarg` errors.","ref":"changelog.html#v0-7-0-2021-05-10"},{"type":"extras","title":"v0.6.3 (2021-02-22) - Changelog","doc":"- Return more verbose errors when finch is configured with bad URLs.","ref":"changelog.html#v0-6-3-2021-02-22"},{"type":"extras","title":"v0.6.2 (2021-02-19) - Changelog","doc":"- Fix incorrect type spec for stream/5\n- Add default transport options for keepalive, timeouts, and nodelay.","ref":"changelog.html#v0-6-2-2021-02-19"},{"type":"extras","title":"v0.6.1 (2021-02-17) - Changelog","doc":"- Update Mint to 1.2.1, which properly handles HTTP/1.0 style responses that close\n  the connection at the same time as sending the response.\n- Update NimblePool to 0.2.4 which includes a bugfix that prevents extra connections\n  being opened.\n- Fix the typespec for Finch.stream/5.\n- Fix assertion that was not actually being called in a test case.","ref":"changelog.html#v0-6-1-2021-02-17"},{"type":"extras","title":"v0.6.0 (2020-12-15) - Changelog","doc":"- Add ability to stream the request body for HTTP/1.x requests.","ref":"changelog.html#v0-6-0-2020-12-15"},{"type":"extras","title":"v0.5.2 (2020-11-10) - Changelog","doc":"- Fix deprecation in nimble_options.","ref":"changelog.html#v0-5-2-2020-11-10"},{"type":"extras","title":"v0.5.1 (2020-10-27) - Changelog","doc":"- Fix crash in http2 pools when a message is received in disconnected state.","ref":"changelog.html#v0-5-1-2020-10-27"},{"type":"extras","title":"v0.5.0 (2020-10-26) - Changelog","doc":"- Add `:max_idle_time` option for http1 pools\n- Optimize http2 connection closing.\n- Use new lazy pools in NimblePool\n- Additional `idle_time` measurements for all http1 connection telemetry","ref":"changelog.html#v0-5-0-2020-10-26"},{"type":"extras","title":"v0.4.0 (2020-10-2) - Changelog","doc":"- Update all dependencies. This includes bug fixes for Mint.","ref":"changelog.html#v0-4-0-2020-10-2"},{"type":"extras","title":"v0.3.2 (2020-09-18) - Changelog","doc":"- Add metadata to connection start telemetry in http/2 pools","ref":"changelog.html#v0-3-2-2020-09-18"},{"type":"extras","title":"v0.3.1 (2020-08-29) - Changelog","doc":"- Add HTTP method to telemetry events\n- BUGFIX - Include query parameters in HTTP/2 requests","ref":"changelog.html#v0-3-1-2020-08-29"},{"type":"extras","title":"v0.3.0 (2020-06-24) - Changelog","doc":"- HTTP/2 support\n- Streaming support for both http/1.1 and http/2 pools\n- New api for building and making requests\n- typespec fixes","ref":"changelog.html#v0-3-0-2020-06-24"},{"type":"extras","title":"v0.2.0 (2020-05-06) - Changelog","doc":"- Response body now defaults to an empty string instead of nil","ref":"changelog.html#v0-2-0-2020-05-06"},{"type":"extras","title":"v0.1.1 (2020-05-04) - Changelog","doc":"- Accepts a URI struct in request/3/4/5/6, Todd Resudek\n- Fix `http_method()` typespec, Ryan Johnson","ref":"changelog.html#v0-1-1-2020-05-04"},{"type":"extras","title":"v0.1.0 (2020-04-25) - Changelog","doc":"- Initial Release","ref":"changelog.html#v0-1-0-2020-04-25"}],"content_type":"text/markdown"}